@@ -1,43 +1,47 @@
+from __future__ import annotations
 import glob
 import json
 import os
 import threading
 from pathlib import Path
 import shutil
-from typing import Dict, List, Optional, Union
-from uuid import uuid4
 import logging
-import pyvis
 from openghg.types import ObjectStoreError
 from openghg.util import read_local_config
 
+
 rlock = threading.RLock()
 
 __all__ = [
     "delete_object",
+    "delete_objects",
     "get_user_objectstore_path",
     "get_tutorial_store_path",
     "get_all_object_names",
     "get_object_names",
     "get_object",
+    "get_object_data_path",
     "set_object",
+    "move_object",
     "set_object_from_json",
     "set_object_from_file",
     "get_object_from_json",
     "exists",
-    "visualise_store",
 ]
 
 logger = logging.getLogger("openghg.objectstore")
 logger.setLevel(logging.DEBUG)  # Have to set level for logger as well as handler
 
 
-def get_readable_buckets() -> Dict[str, str]:
+def get_readable_buckets() -> dict[str, str]:
     """Get a dictionary of readable buckets - {store_name: store_path, ...}
 
     Returns:
         dict: List of readable buckets
     """
+    if os.getenv("OPENGHG_TUT_STORE") is not None:
+        return {"tutorial_store": str(get_tutorial_store_path())}
+
     config = read_local_config()
     object_stores = config["object_store"]
 
@@ -46,7 +50,7 @@ def get_readable_buckets() -> Dict[str, str]:
     }
 
 
-def get_writable_buckets() -> Dict[str, str]:
+def get_writable_buckets() -> dict[str, str]:
     """Get a dictionary of writable buckets - {store_name: store_path, ...}
 
     Returns:
@@ -60,7 +64,7 @@ def get_writable_buckets() -> Dict[str, str]:
     }
 
 
-def get_writable_bucket(name: Optional[str] = None) -> str:
+def get_writable_bucket(name: str | None = None) -> str:
     """Get the path to a writable bucket, passing in the name of a bucket if
     more than one writable bucket available.
 
@@ -77,7 +81,7 @@ def get_writable_bucket(name: Optional[str] = None) -> str:
     if not writable_buckets:
         raise ObjectStoreError("No writable object stores found. Check configuration file.")
 
-    if len(writable_buckets) == 1:
+    if name is None and len(writable_buckets) == 1:
         return next(iter(writable_buckets.values()))
     elif name is not None:
         try:
@@ -113,7 +117,7 @@ def get_user_objectstore_path() -> Path:
     return Path(config["object_store"]["user"]["path"])
 
 
-def get_objectstore_info() -> Dict:
+def get_objectstore_info() -> dict:
     """Read the local config file and return the data of each of the object stores the
     user has access to.
 
@@ -124,7 +128,7 @@ def get_objectstore_info() -> Dict:
     return config["object_store"]
 
 
-def get_all_object_names(bucket: str, prefix: Optional[str] = None, without_prefix: bool = False) -> List:
+def get_all_object_names(bucket: str, prefix: str | None = None, without_prefix: bool = False) -> list:
     """Returns the names of all objects in the passed bucket
 
     Args:
@@ -191,7 +195,22 @@ def delete_object(bucket: str, key: str) -> None:
         pass
 
 
-def get_object_names(bucket: str, prefix: Optional[str] = None) -> List[str]:
+def delete_objects(bucket: str, prefix: str) -> None:
+    """Remove objects with key prefix
+
+    Args:
+        bucket: Bucket path
+        prefix: Key prefix
+    Returns:
+        None
+    """
+    key = Path(bucket, prefix)
+
+    with rlock:
+        shutil.rmtree(path=key, ignore_errors=True)
+
+
+def get_object_names(bucket: str, prefix: str | None = None) -> list[str]:
     """List all the keys in the object store
 
     Args:
@@ -220,6 +239,77 @@ def get_object(bucket: str, key: str) -> bytes:
             raise ObjectStoreError(f"No object at key '{key}'")
 
 
+def move_object(bucket: str, src_key: str, dst_key: str) -> None:
+    """Move data from one place to another
+
+    Args:
+        bucket: Bucket path
+        src_key: Source key
+        dest_key: Destination key
+    Returns:
+        None
+    """
+    src = Path(f"{bucket}/{src_key}._data")
+    dst = Path(f"{bucket}/{dst_key}._data")
+
+    with rlock:
+        shutil.move(src=src, dst=dst)
+
+
+def move_objects(bucket: str, src_prefix: str, dst_prefix: str) -> None:
+    """Move all keys with a certain prefix. Any data in the destination folder
+    will be deleted.
+
+    Args:
+        bucket: Bucket path
+        src_prefix: Source prefix
+        dst_prefix: Destination prefix
+    Returns:
+        None
+    """
+    src = Path(bucket, src_prefix)
+    dst = Path(bucket, dst_prefix)
+
+    with rlock:
+        if dst.exists():
+            shutil.rmtree(dst)
+
+        shutil.move(src=src, dst=dst)
+
+
+def get_object_data_path(bucket: str, key: str) -> Path:
+    """Get path to object data at key in passed bucket.
+
+    Args:
+        bucket: Bucket containing data
+        key: Key for data in bucket
+    Returns:
+        Path to data
+    """
+    filepath = Path(f"{bucket}/{key}._data")
+
+    if filepath.exists():
+        return filepath
+    else:
+        raise ObjectStoreError(f"No object at key '{key}'")
+
+
+def get_object_lock_path(bucket: str, key: str) -> Path:
+    """Get path to object lock file at key in passed bucket.
+
+    Args:
+        bucket: Bucket containing data
+        key: Key for data in bucket
+    Returns:
+        Path to object lock file
+    """
+    lock_path = Path(f"{bucket}/{key}._data.lock_v2")
+    if not lock_path.parent.exists():
+        lock_path.parent.mkdir(parents=True)
+
+    return lock_path
+
+
 def set_object(bucket: str, key: str, data: bytes) -> None:
     """Store data in bucket at key
 
@@ -244,7 +334,7 @@ def set_object(bucket: str, key: str, data: bytes) -> None:
                 f.write(data)
 
 
-def set_object_from_json(bucket: str, key: str, data: Union[str, Dict]) -> None:
+def set_object_from_json(bucket: str, key: str, data: str | dict) -> None:
     """Set JSON data in the object store
 
     Args:
@@ -259,7 +349,7 @@ def set_object_from_json(bucket: str, key: str, data: Union[str, Dict]) -> None:
     set_object(bucket=bucket, key=key, data=data_bytes)
 
 
-def set_object_from_file(bucket: str, key: str, filename: Union[str, Path]) -> None:
+def set_object_from_file(bucket: str, key: str, filename: str | Path) -> None:
     """Set the contents of file at filename to key in bucket
 
     Args:
@@ -272,7 +362,7 @@ def set_object_from_file(bucket: str, key: str, filename: Union[str, Path]) -> N
     set_object(bucket=bucket, key=key, data=open(filename, "rb").read())
 
 
-def get_object_from_json(bucket: str, key: str) -> Dict[str, Union[str, Dict]]:
+def get_object_from_json(bucket: str, key: str) -> dict[str, str | dict]:
     """Return an object constructed from JSON stored at key.
 
     Args:
@@ -281,8 +371,8 @@ def get_object_from_json(bucket: str, key: str) -> Dict[str, Union[str, Dict]]:
     Returns:
         dict: Dictionary
     """
-    data: Union[str, bytes] = get_object(bucket, key).decode("utf-8")
-    data_dict: Dict = json.loads(data)
+    data: str | bytes = get_object(bucket, key).decode("utf-8")
+    data_dict: dict = json.loads(data)
 
     return data_dict
 
@@ -301,7 +391,7 @@ def exists(bucket: str, key: str) -> bool:
     return len(names) > 0
 
 
-def get_bucket(name: Optional[str] = None) -> str:
+def get_bucket(name: str | None = None) -> str:
     """Find and return the local object store path. This will return
     the path to the user's local object store if no name is given.
 
@@ -345,7 +435,26 @@ def clear_object_store() -> None:
         logger.warning("Cannot delete object store.")
 
 
-def query_store() -> Dict:
+def get_folder_size(folder_path: str | Path) -> int:
+    """Get the total size of a folder
+
+    See https://stackoverflow.com/a/75101666/1303032
+
+    Args:
+        folder_path: Path to folder
+    Returns:
+        int: Total size of folder in bytes
+    """
+    total = 0
+    for root, _, files in os.walk(folder_path):
+        for file in files:
+            path = Path(root) / file
+            total += path.stat().st_size
+
+    return total
+
+
+def query_store() -> dict:
     """Create a dictionary that can be used to visualise the object store
 
     Returns:
@@ -374,69 +483,3 @@ def query_store() -> Dict:
         data[d.uuid()] = result
 
     return data
-
-
-def visualise_store() -> pyvis.network.Network:
-    """View the object store using a pyvis force graph.
-
-    This function should only be called from within a notebook
-
-    Returns:
-        pyvis.network.Network
-    """
-    raise NotImplementedError
-    from addict import Dict as aDict
-
-    data = query_store()
-
-    net = pyvis.network.Network("800px", "100%", notebook=True)
-    net.force_atlas_2based()
-
-    # Create the ObsSurface node
-    net.add_node(0, label="Surface Observations", color="#4e79a7", value=5000)
-
-    network_split = aDict()
-
-    for key, value in data.items():
-        # Iterate over Datasources to select the networks
-        network = value["network"]
-        site = value["site"]
-        inlet = value["inlet"]
-        network_split[network][site][inlet][key] = value
-
-    for network, sites in network_split.items():
-        network_name = network.upper()
-        net.add_node(network, label=network_name, color="#59a14f", value=2500)
-        net.add_edge(source=0, to=network)
-
-        # Then we want a subnode for each site
-        for site, site_data in sites.items():
-            # Don't want to use a site here as a site might be in multiple networks
-            site_name = site.upper()
-            site_id = str(uuid4())
-            net.add_node(site_id, label=site_name, color="#e15759", value=1000)
-            net.add_edge(source=network, to=site_id)
-
-            for inlet, inlet_data in site_data.items():
-                inlet_name = str(inlet).lower()
-                inlet_id = str(uuid4())
-                net.add_node(n_id=inlet_id, label=inlet_name, color="#808080", value=500)
-                net.add_edge(source=site_id, to=inlet_id)
-
-                # Now for each site create the datasource nodes
-                for uid, datasource in inlet_data.items():
-                    species = datasource["species"]
-                    instrument = datasource["instrument"].upper()
-
-                    label = f"{species.upper()} {instrument}"
-                    title = "\n".join(
-                        [
-                            f"Site: {site.upper()}",
-                            f"Species : {species.upper()}",
-                            f"Instrument: {instrument}",
-                        ]
-                    )
-                    net.add_node(n_id=uid, label=label, title=title, color="#f28e2b", value=100)
-                    net.add_edge(source=inlet_id, to=uid)
-
-    return net.show("openghg_objstore.html")
