@@ -1,6 +1,7 @@
+from __future__ import annotations
 import logging
 from pathlib import Path
-from typing import Dict, Optional, Union
+from typing import Any, Optional
 
 from numpy import ndarray
 
@@ -8,12 +9,7 @@ from numpy import ndarray
 from openghg.store.base import BaseStore
 from xarray import DataArray
 
-# from openghg.types import multiPathType
-
-__all__ = ["ObsColumn"]
-
-
-ArrayType = Optional[Union[ndarray, DataArray]]
+ArrayType = Optional[ndarray | DataArray]
 
 logger = logging.getLogger("openghg.store")
 logger.setLevel(logging.DEBUG)  # Have to set level for logger as well as handler
@@ -22,29 +18,42 @@ logger.setLevel(logging.DEBUG)  # Have to set level for logger as well as handle
 class ObsColumn(BaseStore):
     """This class is used to process emissions / flux data"""
 
+    _data_type = "column"
     _root = "ObsColumn"
     _uuid = "5c567168-0287-11ed-9d0f-e77f5194a415"
     _metakey = f"{_root}/uuid/{_uuid}/metastore"
 
-    @staticmethod
     def read_file(
-        filepath: Union[str, Path],
-        satellite: Optional[str] = None,
-        domain: Optional[str] = None,
-        selection: Optional[str] = None,
-        site: Optional[str] = None,
-        species: Optional[str] = None,
-        network: Optional[str] = None,
-        instrument: Optional[str] = None,
+        self,
+        filepath: str | Path,
+        species: str,
         platform: str = "satellite",
+        obs_region: Optional[str] = None,
+        satellite: str | None = None,
+        domain: str | None = None,
+        selection: str | None = None,
+        site: str | None = None,
+        network: str | None = None,
+        instrument: str | None = None,
         source_format: str = "openghg",
+        if_exists: str = "auto",
+        save_current: str = "auto",
         overwrite: bool = False,
-    ) -> Optional[Dict]:
+        force: bool = False,
+        compressor: Any | None = None,
+        filters: Any | None = None,
+        chunks: dict | None = None,
+        optional_metadata: dict | None = None,
+    ) -> list[dict]:
         """Read column observation file
 
         Args:
             filepath: Path of observation file
-            satellite: Name of satellite (if relevant)
+            species: Species name or synonym e.g. "ch4"
+            platform: Type of platform. Should be one of:
+                - "satellite"
+                - "site"
+            satellite: Name of satellite (if relevant). Should include satellite OR site.
             domain: For satellite only. If data has been selected on an area include the
                 identifier name for domain covered. This can map to previously defined domains
                 (see openghg_defs "domain_info.json" file) or a newly defined domain.
@@ -52,68 +61,124 @@ class ObsColumn(BaseStore):
                 performed on satellite data. This can be based on any form of filtering, binning etc.
                 but should be unique compared to other selections made e.g. "land", "glint", "upperlimit".
                 If not specified, domain will be used.
-            site : Site code/name (if relevant). Can include satellite OR site.
-            species: Species name or synonym e.g. "ch4"
+            site : Site code/name (if relevant). Should include satellite OR site.
             instrument: Instrument name e.g. "TANSO-FTS"
             network: Name of in-situ or satellite network e.g. "TCCON", "GOSAT"
-            platform: Type of platform. Should be one of:
-                - "satellite"
-                - "site"
             source_format : Type of data being input e.g. openghg (internal format)
-            overwrite: Should this data overwrite currently stored data.
+            if_exists: What to do if existing data is present.
+                - "auto" - checks new and current data for timeseries overlap
+                   - adds data if no overlap
+                   - raises DataOverlapError if there is an overlap
+                - "new" - just include new data and ignore previous
+                - "combine" - replace and insert new data into current timeseries
+            save_current: Whether to save data in current form and create a new version.
+                - "auto" - this will depend on if_exists input ("auto" -> False), (other -> True)
+                - "y" / "yes" - Save current data exactly as it exists as a separate (previous) version
+                - "n" / "no" - Allow current data to updated / deleted
+            overwrite: Deprecated. This will use options for if_exists="new".
+            force: Force adding of data even if this is identical to data stored.
+            compressor: A custom compressor to use. If None, this will default to
+                `Blosc(cname="zstd", clevel=5, shuffle=Blosc.SHUFFLE)`.
+                See https://zarr.readthedocs.io/en/stable/api/codecs.html for more information on compressors.
+            filters: Filters to apply to the data on storage, this defaults to no filtering. See
+                https://zarr.readthedocs.io/en/stable/tutorial.html#filters for more information on picking filters.
+            chunks: Chunking schema to use when storing data. It expects a dictionary of dimension name and chunk size,
+                for example {"time": 100}. If None then a chunking schema will be set automatically by OpenGHG.
+                See documentation for guidance on chunking: https://docs.openghg.org/tutorials/local/Adding_data/Adding_ancillary_data.html#chunking.
+                To disable chunking pass in an empty dictionary.
+            optional_metadata: Allows to pass in additional tags to distinguish added data. e.g {"project":"paris", "baseline":"Intem"}
         Returns:
             dict: Dictionary of datasource UUIDs data assigned to
         """
-        from openghg.store import assign_data, datasource_lookup, load_metastore
-        from openghg.types import ColumnTypes
-        from openghg.util import clean_string, hash_file, load_column_parser
+        # Get initial values which exist within this function scope using locals
+        # MUST be at the top of the function
+        fn_input_parameters = locals().copy()
+
+        from openghg.store.spec import define_standardise_parsers
+        from openghg.util import (
+            clean_string,
+            load_standardise_parser,
+            split_function_inputs,
+            check_if_need_new_version,
+            synonyms,
+        )
 
         # TODO: Evaluate which inputs need cleaning (if any)
-        satellite = clean_string(satellite)
-        site = clean_string(site)
         species = clean_string(species)
+        species = synonyms(species)
+        platform = clean_string(platform)
+
+        if site is None and satellite is None:
+            raise ValueError("Value for 'site' or 'satellite' must be specified")
+        elif site is not None and satellite is not None:
+            raise ValueError("Only one of 'site' or 'satellite' should be specified")
+
+        site = clean_string(site)
+        satellite = clean_string(satellite)
         domain = clean_string(domain)
+        obs_region = clean_string(obs_region)
         network = clean_string(network)
         instrument = clean_string(instrument)
-        platform = clean_string(platform)
+
+        if domain is not None and obs_region is not None:
+            err_msg = f"Only one of 'domain' : {domain} or 'obs_region': {obs_region} should be specified"
+            logger.exception(err_msg)
+            raise ValueError(err_msg)
+        elif domain is not None and obs_region is None:
+            obs_region = domain
+            logger.info(f"Updated 'obs_region' to match 'domain': {domain}")
+        elif obs_region is not None and domain is None:
+            domain = "NOT_SET"
+            logging.info(f"Updated value of 'domain': {domain}")
+
+        # Specify any additional metadata to be added
+        additional_metadata = {}
+
+        if overwrite and if_exists == "auto":
+            logger.warning(
+                "Overwrite flag is deprecated in preference to `if_exists` (and `save_current`) inputs."
+                "See documentation for details of these inputs and options."
+            )
+            if_exists = "new"
+
+        # Making sure new version will be created by default if force keyword is included.
+        if force and if_exists == "auto":
+            if_exists = "new"
+
+        new_version = check_if_need_new_version(if_exists, save_current)
 
         filepath = Path(filepath)
 
+        standardise_parsers = define_standardise_parsers()[self._data_type]
+
         try:
-            source_format = ColumnTypes[source_format.upper()].value
+            source_format = standardise_parsers[source_format.upper()].value
         except KeyError:
             raise ValueError(f"Unknown data type {source_format} selected.")
 
         # Load the data retrieve object
-        parser_fn = load_column_parser(source_format=source_format)
+        parser_fn = load_standardise_parser(data_type=self._data_type, source_format=source_format)
 
-        obs_store = ObsColumn.load()
+        # Get current parameter values and filter to only include function inputs
+        fn_current_parameters = locals().copy()  # Make a copy of parameters passed to function
+        fn_input_parameters = {key: fn_current_parameters[key] for key in fn_input_parameters}
 
-        # Load in the metadata store
-        metastore = load_metastore(key=obs_store._metakey)
+        _, unseen_hashes = self.check_hashes(filepaths=filepath, force=force)
 
-        file_hash = hash_file(filepath=filepath)
-        if file_hash in obs_store._file_hashes and not overwrite:
-            logger.warning(
-                "This file has been uploaded previously with the filename : "
-                f"{obs_store._file_hashes[file_hash]} - skipping."
-            )
-            return None
+        if not unseen_hashes:
+            return [{}]
 
-        # Define parameters to pass to the parser function
-        param = {
-            "data_filepath": filepath,
-            "satellite": satellite,
-            "domain": domain,
-            "selection": selection,
-            "site": site,
-            "species": species,
-            "network": network,
-            "instrument": instrument,
-            "platform": platform,
-        }
+        filepath = next(iter(unseen_hashes.values()))
 
-        obs_data = parser_fn(**param)
+        if chunks is None:
+            chunks = {}
+
+        # Define parameters to pass to the parser function and remaining keys
+        parser_input_parameters, additional_input_parameters = split_function_inputs(
+            fn_input_parameters, parser_fn
+        )
+
+        obs_data = parser_fn(**parser_input_parameters)
 
         # TODO: Add in schema and checks for ObsColumn
         # # Checking against expected format for ObsColumn
@@ -125,26 +190,34 @@ class ObsColumn(BaseStore):
         # this could be "site" or "satellite" keys.
         # platform = list(obs_data.keys())[0]["metadata"]["platform"]
 
-        required = ("satellite", "selection", "domain", "site", "species", "network")
-        lookup_results = datasource_lookup(
-            metastore=metastore, data=obs_data, required_keys=required, min_keys=3
-        )
+        # Check to ensure no required keys are being passed through optional_metadata dict
+        self.check_info_keys(optional_metadata)
+        if optional_metadata is not None:
+            additional_metadata.update(optional_metadata)
+
+        # Mop up and add additional keys to metadata which weren't passed to the parser
+        obs_data = self.update_metadata(obs_data, additional_input_parameters, additional_metadata)
 
         data_type = "column"
-        datasource_uuids = assign_data(
-            data_dict=obs_data,
-            lookup_results=lookup_results,
-            overwrite=overwrite,
+        datasource_uuids = self.assign_data(
+            data=obs_data,
+            if_exists=if_exists,
+            new_version=new_version,
             data_type=data_type,
+            compressor=compressor,
+            filters=filters,
         )
 
-        obs_store.add_datasources(uuids=datasource_uuids, data=obs_data, metastore=metastore)
+        # TODO: MAY NEED TO ADD BACK IN OR CAN DELETE
+        # update_keys = ["start_date", "end_date", "latest_version"]
+        # obs_data = update_metadata(data_dict=obs_data, uuid_dict=datasource_uuids, update_keys=update_keys)
+
+        # obs_store.add_datasources(
+        #     uuids=datasource_uuids, data=obs_data, metastore=metastore, update_keys=update_keys
+        # )
 
         # Record the file hash in case we see this file again
-        obs_store._file_hashes[file_hash] = filepath.name
-
-        obs_store.save()
-        metastore.close()
+        self.store_hashes(unseen_hashes)
 
         return datasource_uuids
 
