@@ -3,8 +3,12 @@ from ._enum import (
     ColumnTypes,
     ColumnSources,
     ObsTypes,
-    EmissionsTypes,
-    EmissionsDatabases,
+    FluxTimeseriesTypes,
+    FluxTypes,
+    FluxDatabases,
+    FootprintTypes,
+    BoundaryConditions,
+    EulerianModelTypes,
     DataTypes,
 )
 from ._errors import (
@@ -14,11 +18,19 @@ from ._errors import (
     FunctionError,
     ObjectStoreError,
     DatasourceLookupError,
+    DatasourceCombineError,
     EncodingError,
     MutexTimeoutError,
     RequestBucketError,
     SearchError,
+    ParseError,
     AttrMismatchError,
+    MetadataFormatError,
+    DataOverlapError,
+    ConfigFileError,
+    MetastoreError,
+    ZarrStoreError,
+    KeyExistsError,
     construct_xesmf_import_error,
 )
 from ._types import (
@@ -30,4 +42,9 @@ from ._types import (
     ArrayLikeMatch,
     XrDataLike,
     XrDataLikeMatch,
+    TimePeriod,
+    HasMetadataAndData,
+    MetadataAndData,
+    Comparable,
+    convert_to_list_of_metadata_and_data,
 )
